@@ -6,6 +6,7 @@ import sqlalchemy
 from sqlalchemy.engine import create_engine
 from sqlalchemy import MetaData, Table
 from converter.vertica.custom_insert import vertica_insert
+from converter.vertica.merge import vertica_merge
 
 warnings.filterwarnings("ignore")
 
@@ -45,7 +46,7 @@ def get_default_arg_sql_alchemy(column):
 
 class DataBaseWorker:
     def __init__(
-        self, sql_credentials, db, debug=True, tables=None, table_checker=True
+            self, sql_credentials, db, debug=True, tables=None, table_checker=True
     ):
         """
         :param sql_credentials:
@@ -74,45 +75,13 @@ class DataBaseWorker:
             if key not in ["database", "schema", "user", "host", "password", "port"]:
                 additional_args[key] = self.sql_credentials[self.db][key]
         self.engine = create_engine(uri_sql_alchemy, **additional_args)
-        self.log("try to connect...")
-        self.conn = self.engine.connect()
-        self.schema = None
-        self.meta = None
-        self.init_meta()
-        self.log("connecting is successfull")
-
-    def init_meta(self, tables=None):
-        """
-        reinit meta objects
-        :return:
-        """
         if "schema" in self.cred:
             self.schema = self.cred["schema"]
-            self.meta = MetaData(bind=self.engine, schema=self.schema)
         else:
-            self.meta = MetaData(bind=self.engine)
             self.schema = self.cred["database"]
-
-        tables = tables or self.tables
-        if len(tables) > 0 and self.table_checker:
-            try:
-                self.meta.reflect(only=tables)
-            except sqlalchemy.exc.InvalidRequestError:
-                raise sqlalchemy.exc.InvalidRequestError(
-                    "One or more file didn't find in database {}. Critical exception"
-                )
-        elif len(tables) > 0 and not self.table_checker:
-            try:
-                self.meta.reflect(only=tables)
-            except sqlalchemy.exc.InvalidRequestError:
-                self.log(
-                    "One or more file didn't find in database {}. Load table later".format(
-                        self.db
-                    )
-                )
-                self.meta = None
-        else:
-            self.meta.reflect()
+        self.log("try to connect...")
+        self.conn = self.engine.connect()
+        self.log("connecting is successfull")
 
     def __check_dialect(self):
         if self.db not in base_converter_params.keys():
@@ -123,6 +92,34 @@ class DataBaseWorker:
             )
         return True
 
+    @property
+    def meta(self, tables=None):
+        if "schema" in self.cred:
+            meta = MetaData(bind=self.engine, schema=self.schema)
+        else:
+            meta = MetaData(bind=self.engine)
+        tables = tables or self.tables
+        if len(tables) > 0 and self.table_checker:
+            try:
+                meta.reflect(only=tables)
+            except sqlalchemy.exc.InvalidRequestError:
+                raise sqlalchemy.exc.InvalidRequestError(
+                    "One or more file didn't find in database {}. Critical exception"
+                )
+        elif len(tables) > 0 and not self.table_checker:
+            try:
+                meta.reflect(only=tables)
+            except sqlalchemy.exc.InvalidRequestError:
+                self.log(
+                    "One or more file didn't find in database {}. Load table later".format(
+                        self.db
+                    )
+                )
+                meta = None
+        else:
+            meta.reflect()
+        return meta
+
     @property
     def dialect(self):
         return base_converter_params[self.db]["dialect"]
@@ -139,9 +136,6 @@ class DataBaseWorker:
     def __repr__(self):
         return "DataBaseWorker_{}".format(self.db)
 
-    def __del__(self):
-        self.conn.close()
-
     def log(self, text):
         if self.debug:
             logging.info(self.__str__ + ": " + text)
@@ -155,26 +149,90 @@ class DataBaseWorker:
             )
         return True
 
-    def insert_vertica(self, table_name, data):
+    def insert_vertica(self, table_name, data, schema=None):
         """
         custom insert for vertica using json
+        :param schema:  schema to uplaad
         :param table_name:
         :param data: data to insert
-        :return:
+        :return: nothing
         """
+        if self.db != "vertica":
+            raise ModuleNotFoundError("Available only for vertica data source")
         cursor = self.engine.raw_connection().cursor()
         keys_data = list(data[0].keys())
         columns = self.get_columns(table_name)
+        keys_to_remove = list(set(keys_data) - set(columns))
         keys_data = [key for key in keys_data if key in columns]
-        fields_json = [key for key in keys_data if columns[key] in ("long varbinary", "blob")]
+        for item in data:
+            for key in keys_to_remove:
+                del item[key]
+        fields_json = [
+            key
+            for key in keys_data
+            if key in columns and columns[key] in ("long varbinary", "blob")
+        ]
         vertica_insert(
             cursor=cursor,
-            target=self.schema + "." + table_name,
+            target=(schema or self.schema) + "." + table_name,
             fields_json=fields_json,
             fields=keys_data,
             data=data,
         )
 
+    def insert_merge_vertica(
+            self,
+            table_name,
+            data,
+            staging_schema=None,
+            schema=None,
+            staging_table_suffix=None,
+            key_fields=None,
+            filter_merge_fields=None,
+    ):
+        """
+        Insert and update data in target table using temporary staging table, only for vertica
+        :param filter_merge_fields:
+        :param key_fields:
+        :param table_name:
+        :param data:
+        :param staging_schema:
+        :param schema:
+        :param staging_table_suffix:
+        :return: Nothing
+        """
+        if key_fields is None:
+            key_fields = ["id"]
+        if self.db != "vertica":
+            raise ModuleNotFoundError("Available only for vertica data source")
+        schema = schema or self.schema
+        staging_table_name = "{staging_schema}.{table_name}{staging_table_suffix}".format(
+            staging_schema=staging_schema,
+            table_name=table_name,
+            staging_table_suffix=staging_table_suffix or "",
+        )
+        sql_create_staging = "create table {staging_table_name} like {schema}.{table_name}".format(
+            staging_table_name=staging_table_name, table_name=table_name, schema=schema
+        )
+        self.log(sql_create_staging)
+        self.conn.execute(sql_create_staging)
+        try:
+            self.insert_vertica(table_name, data, schema=staging_schema)
+            vertica_merge(
+                conn=self.conn,
+                table_name=schema + "." + table_name,
+                staging_table_name=staging_table_name,
+                fields=self.get_columns(table_name),
+                key_fields=key_fields,
+                filter_merge_fields=filter_merge_fields,
+            )
+        finally:
+            sql_drop = "drop table {staging_table_name} cascade".format(
+                staging_table_name=staging_table_name
+            )
+            self.log(sql_drop)
+            self.conn.execute(sql_drop)
+
     def get_table_schema(self, table_name):
         columns_name = [
             "column_name",
