@@ -1,10 +1,8 @@
 import json
-import datetime
 
 
 def myconverter(o):
-    if isinstance(o, datetime.date) or isinstance(o, datetime.datetime):
-        return o.__str__()
+    return o.__str__()
 
 
 def write_list_as_json_str(fields_json, rows):
@@ -26,3 +24,4 @@ def vertica_insert(cursor, target, fields, fields_json, data):
     ).format(target, ",".join(['"' + v + '"' for v in fields]))
     data_str = write_list_as_json_str(fields_json, data)
     cursor.copy(copy_statement, data_str)
+    cursor.close()
